"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = __importDefault(require("axios"));
const backtrace_service_1 = require("backtrace-service");
// import { Response } from 'express';
const main_1 = require("../main");
const log = backtrace_service_1.Logger.getLogger();
async function send(submitUrl, metrics, reports, reportData, format, correlation, universe = "", defectProjectName, batchIdentifier) {
    let count = reports.length;
    log.debug(`Sending ${count} reports in batch #${batchIdentifier}.`);
    const respObj = {
        resource: submitUrl,
        received: count,
        status: 'ok',
        batchId: batchIdentifier,
    };
    if (count === 0) {
        return respObj;
    }
    const promises = [];
    // tslint:disable-next-line: prefer-for-of
    for (let i = 0; i < reports.length; i++) {
        const opts = {
            headers: {
                'Content-Type': `multipart/form-data; boundary=${reports[i].getBoundary()}`,
            },
            httpsAgent: main_1.httpsAgent,
            httpAgent: main_1.httpAgent,
            maxContentLength: Infinity,
        };
        promises.push(axios_1.default.post(submitUrl, reports[i], opts));
    }
    await Promise.all(promises)
        .then(function (resps) {
        const correlationRequest = {
            universe,
            defectProject: defectProjectName,
        };
        // cache defect in defect table upon successful submission to coronerd
        resps.forEach(async (resp, index) => {
            var _a, _b;
            if (((_b = (_a = resp) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.response) === 'ok' && reportData[index]) {
                await correlation.insertDefect(reportData[index], correlationRequest);
            }
        });
        metrics.addMetric(`created-${format}`, count);
        respObj.status = 'ok';
    })
        .catch(function (err) {
        // err from Promise.all has a LOT of stuff on it (req, res, json, etc..)
        // Here I re-make the error with just essentials to avoid spamming logs.
        const compactError = new Error(err.message);
        compactError.stack = err.stack; // keep for a better BT error report :)
        log.error(compactError);
        metrics.addMetric(`failure-${format}`, count);
        respObj.status = 'error';
    });
    return respObj;
}
exports.send = send;
exports.default = send;
//# sourceMappingURL=send.js.map