"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
// tslint:disable-next-line: no-var-requires
const xml2js_1 = __importDefault(require("xml2js"));
const form_data_1 = __importDefault(require("form-data"));
const fs_1 = __importStar(require("fs"));
const path = __importStar(require("path"));
const zlib_1 = require("zlib");
const stream_1 = require("stream");
const util_1 = require("util");
const pipe = util_1.promisify(stream_1.pipeline);
// Take last 2 sections of filepath.
function reduceFilepath(filepath) {
    return filepath.split('/').slice(-2).join('/');
}
exports.reduceFilepath = reduceFilepath;
async function parseAsync(data) {
    const parser = new xml2js_1.default.Parser({ explicitArray: false, mergeAttrs: true });
    return await parser.parseStringPromise(data);
}
exports.parseAsync = parseAsync;
function getDefectClassifiers(type, classifierFields) {
    const classifiers = new Set(['defect']);
    if (type) {
        if (classifierFields[type]) {
            classifierFields[type].forEach(cf => {
                classifiers.add(cf); // add existing classifiers
            });
        }
        classifiers.add(type); // add the rule-type itself
    }
    return Array.from(classifiers);
}
exports.getDefectClassifiers = getDefectClassifiers;
// File compression
async function do_gzip(input, output) {
    const gzip = zlib_1.createGzip();
    const source = fs_1.createReadStream(input);
    const destination = fs_1.createWriteStream(output);
    await pipe(source, gzip, destination);
}
// Convert report/attachment objects to FormData for submission
function reportToFormData(obj, log) {
    const formData = new form_data_1.default();
    const json = JSON.stringify(obj.report);
    formData.append('upload_file', json, 'upload_file.json');
    if (obj.attachmentPath && fs_1.default.existsSync(obj.attachmentPath)) {
        const name = path.basename(obj.attachmentPath);
        formData.append(`attachment_${name}`, fs_1.default.createReadStream(obj.attachmentPath), name);
    }
    return formData;
}
exports.reportToFormData = reportToFormData;
// Latency in microseconds from a given startTime
function getLatency(startTime) {
    const nanoLatency = process.hrtime.bigint() - startTime;
    const microLatency = nanoLatency / BigInt(1e3);
    return microLatency.toString(10);
}
exports.getLatency = getLatency;
function queryForCrashesSince(time) {
    return {
        group: ['fingerprint'],
        fold: {
            fingerprint: [['head']],
        },
        filter: [
            {
                timestamp: [['at-least', time]],
                classifiers: [['not-contains', "defect"]],
            }
        ]
    };
}
exports.queryForCrashesSince = queryForCrashesSince;
;
const queryParamRegex = /[?&]([^=#]+)=([^&#]*)/g;
function getQueryParams(url) {
    const params = {};
    let match;
    while (match = queryParamRegex.exec(url)) {
        params[match[1]] = match[2];
    }
    return params;
}
exports.getQueryParams = getQueryParams;
;
function getUniverseType(submitUrl) {
    if (submitUrl.indexOf("https://submit") > -1 || submitUrl.indexOf("http://submit") > -1) {
        return "TYPEB";
    }
    return "TYPEA";
}
exports.getUniverseType = getUniverseType;
const startMarkerTypeA = "://";
function getUniverseTokenFromTypeA(submitUrl) {
    let startIndex = submitUrl.indexOf(startMarkerTypeA);
    if (startIndex < 0)
        return {};
    startIndex += startMarkerTypeA.length;
    const newStr = submitUrl.slice(startIndex);
    const endIndex = newStr.indexOf(".");
    const universe = newStr.slice(0, endIndex);
    return {
        token: getQueryParams(submitUrl).token,
        universe,
    };
}
exports.getUniverseTokenFromTypeA = getUniverseTokenFromTypeA;
const startMarkerTypeB = ".backtrace.io/";
const endMarkerTypeB = "/json";
function getUniverseTokenFromTypeB(submitUrl) {
    let startIndex = submitUrl.indexOf(startMarkerTypeB);
    if (startIndex < 0)
        return {};
    startIndex += startMarkerTypeB.length;
    const newStr = submitUrl.slice(startIndex);
    const endIndex = newStr.indexOf("/");
    const universe = newStr.slice(0, endIndex);
    const token = newStr.slice(endIndex + 1, newStr.indexOf(endMarkerTypeB));
    return {
        token,
        universe,
    };
}
exports.getUniverseTokenFromTypeB = getUniverseTokenFromTypeB;
function getUniverseToken(submitUrl) {
    switch (getUniverseType(submitUrl)) {
        case "TYPEA":
            return getUniverseTokenFromTypeA(submitUrl);
        case "TYPEB":
            return getUniverseTokenFromTypeB(submitUrl);
        default:
            return;
    }
}
exports.getUniverseToken = getUniverseToken;
//# sourceMappingURL=utilities.js.map