const fs = require('fs');
const path = require('path');
const BacktraceService = require('backtrace-service').BacktraceService;
const Logger = require('backtrace-service').Logger;

const log = Logger.getLogger();

const config = BacktraceService.readConfiguration("sca", 6449, undefined, true);
const configDb = config.database;

// Check if the db path from config exists on machine. If not, create it now.
const directory = path.dirname(configDb.connection.filename);
fs.mkdir(directory, { recursive: true }, (err) => {
  if (err) { 
    log.error(`Error: Database directory ${directory} cannot be created.`);
    throw err;
  }
});

module.exports = {
  development: {
    client: 'sqlite3',
    connection: {
      filename: './dev.sqlite3',
    },
    migrations: {
      directory: './migrations',
    },
    useNullAsDefault: true,
  },
  production: Object.assign(
    {
      migrations: {
        directory: './migrations',
      },
      useNullAsDefault: true,
    },
    configDb,
  ),
  test: {
    client: 'sqlite3',
    connection: {
      filename: './test.sqlite3',
    },
    migrations: {
      directory: './migrations',
    },
    useNullAsDefault: true,
  }
};
