module.exports.up = function(knex) {
    return knex.schema
    .dropTable('correlation')
    .createTable('crash', function(crash) {
        crash.text('fingerprint').notNullable();
        crash.text('universe').notNullable();
        crash.text('project').notNullable();
        crash.text('build_id').defaultTo('*');
        crash.primary(['fingerprint', 'build_id'], 'primary_fingerprint_build_id');
    })
    .createTable('defect', function(defect) {
        defect.text('fingerprint').notNullable();
        defect.text('universe').notNullable();
        defect.text('project').notNullable();
        defect.text('build_id').defaultTo('*');
        defect.text('error_message');
        defect.primary(['fingerprint', 'build_id'], 'primary_fingerprint_build_id');
    })
    .createTable('correlation', function(correlation) {
        correlation.integer('crash').references('crash.rowid');
        correlation.integer('defect').references('defect.rowid');
        correlation.index(['crash'], 'correlation_crash_idx');
        correlation.index(['defect'], 'correlation_defect_idx');
    })
  };
  
  module.exports.down = function(knex) {
    return knex.schema
    // drop newly created tables
    .dropTable('correlation')
    .dropTable('defect')
    .dropTable('crash')
    // recreate the former table
    .createTable('correlation', function(t) {
        t.increments('id')
            .unsigned()
            .primary()
            .defaultTo(1);
        t.timestamp('created_at').defaultTo(knex.fn.now());
        t.string('fingerprint').notNullable();
        t.string('universe').notNullable();
        t.string('defects', 10000).notNullable();
        t.integer("defect_count").defaultTo(0);
        t.string('crashProject');
        t.string('defectProject');
        t.string('build');
        t.string('latency');
        t.index(['universe', 'crashProject', 'defectProject', 'fingerprint'], 'correlationIndex');
    });
  }