"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = __importStar(require("fs"));
var path_1 = require("path");
var source_scan_1 = require("source-scan");
/**
 * Analyse stack trace generated by exception
 * Create Stack Frames and find calling library/program informaiton
 */
var BacktraceStackTrace = /** @class */ (function () {
    function BacktraceStackTrace(error) {
        this.error = error;
        this.fault = true;
        this.name = 'main';
        this.stack = [];
        this.sourceCodeInformation = {};
        this.callingModulePath = '';
        this.stackLineRe = /\s+at (.+) \((.+):(\d+):(\d+)\)/;
        this.requestedSourceCode = {};
        this.tabWidth = 8;
        this.contextLineCount = 200;
    }
    BacktraceStackTrace.prototype.setSourceCodeOptions = function (tabWidth, contextLineCount) {
        this.tabWidth = tabWidth;
        this.contextLineCount = contextLineCount;
    };
    /**
     * Get calling module path
     */
    BacktraceStackTrace.prototype.getCallingModulePath = function () {
        if (!this.stack || this.stack.length === 0) {
            return undefined;
        }
        // handle a situation when every one stack frame is from node_modules
        if (!this.callingModulePath) {
            this.callingModulePath = this.stack[0].sourceCode;
        }
        return this.callingModulePath;
    };
    /**
     * Get Json data from Stack trace object
     */
    BacktraceStackTrace.prototype.toJson = function () {
        return {
            name: this.name,
            fault: this.fault,
            stack: this.stack,
        };
    };
    /**
     * Get source code information
     */
    BacktraceStackTrace.prototype.getSourceCode = function () {
        return this.sourceCodeInformation;
    };
    /**
     * Start parsing stack frames
     */
    BacktraceStackTrace.prototype.parseStackFrames = function () {
        return __awaiter(this, void 0, void 0, function () {
            var stackTrace, lines, backtracePath;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        stackTrace = this.error.stack;
                        if (!stackTrace) {
                            return [2 /*return*/];
                        }
                        lines = stackTrace.split('\n').slice(1);
                        backtracePath = path_1.join('node_modules', 'backtrace-node');
                        lines.forEach(function (line) {
                            var match = line.match(_this.stackLineRe);
                            if (!match || match.length < 4) {
                                return;
                            }
                            var backtraceLibStackFrame = match[2].indexOf(backtracePath) !== -1;
                            if (backtraceLibStackFrame) {
                                return;
                            }
                            var stackFrame = {
                                funcName: match[1],
                                sourceCode: match[2],
                                library: match[2],
                                line: parseInt(match[3], 10),
                                column: parseInt(match[4], 10),
                            };
                            _this.addSourceRequest(stackFrame);
                            if (_this.isCallingModule(stackFrame)) {
                                _this.callingModulePath = stackFrame.sourceCode;
                            }
                            _this.stack.push(stackFrame);
                        });
                        // read source code information after reading all existing stack frames from stack trace
                        return [4 /*yield*/, this.readSourceCode()];
                    case 1:
                        // read source code information after reading all existing stack frames from stack trace
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    BacktraceStackTrace.prototype.addSourceRequest = function (stackFrame) {
        // ignore not existing stack frames
        if (!fs.existsSync(stackFrame.sourceCode)) {
            return;
        }
        // add source code to existing list. Otherwise create empty array
        this.requestedSourceCode[stackFrame.sourceCode] = this.requestedSourceCode[stackFrame.sourceCode] || [];
        this.requestedSourceCode[stackFrame.sourceCode].push({
            line: stackFrame.line,
            column: stackFrame.column,
        });
    };
    BacktraceStackTrace.prototype.readSourceCode = function () {
        return __awaiter(this, void 0, void 0, function () {
            var _a, _b, _i, key, element, minLine, maxLine, i, item, parameter, res;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        _a = [];
                        for (_b in this.requestedSourceCode)
                            _a.push(_b);
                        _i = 0;
                        _c.label = 1;
                    case 1:
                        if (!(_i < _a.length)) return [3 /*break*/, 4];
                        key = _a[_i];
                        if (!this.requestedSourceCode.hasOwnProperty(key)) return [3 /*break*/, 3];
                        element = this.requestedSourceCode[key];
                        minLine = element[0].line;
                        maxLine = minLine;
                        for (i = 1; i < element.length; i += 1) {
                            item = element[i];
                            minLine = Math.min(minLine, item.line);
                            maxLine = Math.max(maxLine, item.line);
                        }
                        parameter = {
                            startLine: Math.max(minLine - this.contextLineCount + 1, 0),
                            endLine: maxLine + this.contextLineCount,
                            filePath: key,
                        };
                        return [4 /*yield*/, this.getSourceCodeInformation(parameter)];
                    case 2:
                        res = _c.sent();
                        this.sourceCodeInformation[key] = res;
                        _c.label = 3;
                    case 3:
                        _i++;
                        return [3 /*break*/, 1];
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    BacktraceStackTrace.prototype.getSourceCodeInformation = function (parameter) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                return [2 /*return*/, new Promise(function (res, rej) {
                        source_scan_1.scanFile(parameter, function (err, buff) {
                            if (err) {
                                rej(err);
                                return;
                            }
                            res({
                                path: parameter.filePath,
                                startLine: parameter.startLine + 1,
                                startColumn: 1,
                                text: buff.toString('utf8'),
                                tabWidth: _this.tabWidth,
                            });
                        });
                    })];
            });
        });
    };
    BacktraceStackTrace.prototype.isCallingModule = function (stackFrame) {
        return (!this.callingModulePath && fs.existsSync(stackFrame.sourceCode) && !stackFrame.sourceCode.includes('node_modules'));
    };
    return BacktraceStackTrace;
}());
exports.BacktraceStackTrace = BacktraceStackTrace;
//# sourceMappingURL=backtraceStackTrace.js.map